// stubTest.cpp : Defines the entry point for the application.
/*!
 * \file stubTest.cpp
 * \ingroup stubTest
 */

#include "StdAfx.h"
#include "process.h"
#include "..\win2vcc\cmnhdr.h"

#define CLIENTNAME TEXT("Client0")

BYTE palette0[256*3];	//!< An array of palette bytes
BYTE screen0[240*320];	//!< An array of screen bytes

BYTE palette1[256*3];	//!< An array of screen bytes
BYTE screen1[240*320];	//!< An array of palette bytes

MouseClient mouseIn( CLIENTNAME ); //!< The mouse Pipe
ScreenClient screenOut( CLIENTNAME ); //!< The screen Pipe

DWORD WINAPI screenThread(LPVOID);

/*! StubTest entry point function. */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)

{
	AllocConsole();
	HANDLE console = GetStdHandle(STD_OUTPUT_HANDLE);
	SetConsoleTitle(TEXT("VCC pipe IO Stub"));
	
	// Initialise the 2 screen buffers and palettes

	int i;
	for(i=0; i<256*3;){
		palette0[i] = i % 256;
		i++;
		palette0[i] = i % 128;
		i++;
		palette0[i] = (i % 60) * 4;
		i++;
	}
	
	for(i=0; i<240*320; i++){
		screen0[i] = i % 256;
	}

	
    srand( (unsigned)time( NULL ) );
	for(i=0; i<256*3;){
		palette1[i] = rand() % 256;
		i++;
		palette1[i] = rand() % 128 + 64;
		i++;
		palette1[i] = i % 256;
		i++;
	}
	
	for(i=0; i<240*320; i++){
		screen1[i] = rand() % 256;
	}

	CHString msgString;

	switch( mouseIn.connect() ){
	case PC_ALREADY_CONNECTED:
	case PC_CONNECTED:
		msgString.Format(TEXT("Mouse Connected\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		break;
	case PC_FAILED:
		msgString.Format(TEXT("Error Connecting Mouse\nExit in 4 seconds\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		Sleep(4000);
		return 1;
		break;
	case PC_BUSY:	
		msgString.Format(TEXT("Error - Mouse Pipe already connected\nExit in 4 seconds\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		Sleep(4000);
		return 1;
		break;
	}

	switch( screenOut.connect() ){
	case PC_ALREADY_CONNECTED:
	case PC_CONNECTED:
		msgString.Format(TEXT("Screen Connected\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		break;
	case PC_FAILED:
		msgString.Format(TEXT("Error Connecting Screen\nExit in 4 seconds\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		Sleep(4000);
		return 1;
		break;
	case PC_BUSY:	
		msgString.Format(TEXT("Error - Screen Pipe already connected\nExit in 4 seconds\n"));
		WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		Sleep(4000);
		return 1;
		break;
	}


	chBEGINTHREADEX(NULL, 0, &screenThread, NULL, 0, NULL);

	mouseMessage msg;
	bool exit = false;
	for(i = 1; i>0 && !exit;){
		switch( mouseIn.deQueueMessage(&msg) ){
		case MI_SUCCESS:
			msgString.Format(TEXT("%d: {%.3i,%.3i} %i\n"), i++, msg.x, msg.y, msg.button);
			WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
			if( i % 9 == 0 ){ // simulate slow vcc
				Sleep(1000);
			}
			break;
		case MI_NO_MESSAGE:
			Sleep(100);
			break;
		case MI_NOT_CONNECTED:
			msgString.Format(TEXT("Server Disconnected\n"));
			WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
		case MI_UNKNOWN_ERROR:
		default:
			msgString.Format(TEXT("Error getting Message\n"));
			WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);
			Sleep(5000);
			exit = true;
		}
	}

	mouseIn.disconnect();
	screenOut.disconnect();

	FreeConsole();
	CloseHandle(console);
	return 0;
}

/*! The screen pipe test thread */
DWORD WINAPI screenThread(LPVOID)
{
	while( true ){
		if(!screenOut.write(240, 320, 256, 8, (RGBTRIPLE*) palette0, screen0))
			break;
		Sleep(1000);

		if(!screenOut.write(240, 320, 256, 8, (RGBTRIPLE*) palette1, screen1))
			break;
		Sleep(1000);

		if(!screenOut.write(240, 320, 256, 8, (RGBTRIPLE*) palette0, screen1))
			break;
		Sleep(1000);

		if(!screenOut.write(240, 320, 256, 8, (RGBTRIPLE*) palette1, screen0))
			break;
		Sleep(1000);
	}
	HANDLE console = GetStdHandle(STD_OUTPUT_HANDLE);
	CHString msgString;
	msgString.Format(TEXT("\nSCREEN DIDN'T WRITE\n"));
	WriteConsole(console, msgString, msgString.GetLength(), NULL, NULL);

	return true;
}
