// MouseClient.cpp: implementation of the MouseClient class.
/*! \file MouseClient.cpp
*/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MouseClient.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*!
 * @param name : A TString used to generate unique names for the
 * various kernel objects
 */
MouseClient::MouseClient(const TString &name)
:PClient( TString( TEXT("MOUSE") ) + name, false)
{
	semName = TString( TEXT("MOUSESEM") ) + name;
	hasMessage = NULL;
}

/*!
 * Object Destructor
 */
MouseClient::~MouseClient()
{
	if(hasMessage != NULL)
		CloseHandle(hasMessage);
}

//////////////////////////////////////////////////////////////////////
// Member Functions
//////////////////////////////////////////////////////////////////////

DWORD MouseClient::connect()
{	
	DWORD success = PClient::connect();

	if( PC_CONNECTED == success ) {
		hasMessage = OpenSemaphore(SEMAPHORE_ALL_ACCESS, false, semName);
		if( hasMessage == NULL ){
			PClient::disconnect();
			success = PC_FAILED;
		}
	}

	return success;
}

void MouseClient::disconnect()
{
	if(hasMessage != NULL)
		CloseHandle(hasMessage);
	PClient::disconnect();
}

/*!
 * @param message : pointer to a mouseMessage structure
 * @return DWORD : error status
 */
DWORD MouseClient::deQueueMessage(mouseMessage *message)
{
	if( !isConnected() ){
		return MI_NOT_CONNECTED;
	}
	
	DWORD numBytesRead;

	if( message == NULL ) {
		return MI_INVALID_PTR;

	} else if( WAIT_OBJECT_0 == WaitForSingleObject(hasMessage, 0) ) {

		if( ReadFile( 
				hPipe,					// Read from
				message,				// data buffer
				sizeof(mouseMessage),	// number of bytes to read
				&numBytesRead,			// number of bytes read
				NULL) ) {

			if( numBytesRead != sizeof(mouseMessage) ) {
				return MI_UNKNOWN_ERROR;
			} else {
				return MI_SUCCESS;
			}
		} else if(ERROR_READ_FAULT == GetLastError()) {
			disconnect();
			/* \todo decide what to do (throw exception?) */
		} 
		return MI_UNKNOWN_ERROR;
	} else {
		return MI_NO_MESSAGE;
	}
}
