// ScreenClient.cpp: implementation of the ScreenClient class.
/*! \file ScreenClient.cpp
*/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ScreenClient.h"

//////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////

/*!
 * @param name : A TString used to generate unique names for the
 * various kernel objects
 */
ScreenClient::ScreenClient(const TString &name)
:PClient(TString( TEXT("SCREEN") ) + name, true)
{}

//////////////////////////////////////////////////////////////////////
// Member Functions
//////////////////////////////////////////////////////////////////////

/*!
 * Writes a screen buffer to the pipe
 *
 * @param width : The screen buffers width
 * @param height : The screen bufers signed height
 * @param clrUsed : The number of colours in the palette
 * @param bitCount : The number of bits per pixel
 * @param palette : An array of RGBTRIPLE structures
 * @param imageBits : An array of bytes representing pixels (index or 
 * colour values depending on bpp)
 */
bool ScreenClient::write(const int width, 
					  const int height, 
					  const int clrUsed, 
					  const int bitCount, 
					  const RGBTRIPLE* palette, 
					  const LPBYTE imageBits)
{
	screenInfo header;
	DWORD numBytesWritten;
	bool fOk;

	if( (palette == NULL && clrUsed > 0) || imageBits == NULL )
		return false;

	header.hdrSize = sizeof(screenInfo);
	header.width = width;
	header.height = height;
	header.clrUsed = clrUsed;
	header.bitCount = bitCount;

	fOk = !WriteFile(hPipe,
			&header,
			sizeof(screenInfo),
			&numBytesWritten,
			NULL);

	if( fOk || (numBytesWritten != sizeof(screenInfo)) )
		return false;

	if(palette != NULL){
		fOk = !WriteFile(hPipe,
			palette,
			sizeof(RGBTRIPLE) * clrUsed,
			&numBytesWritten,
			NULL);

		if( fOk || (numBytesWritten != (sizeof(RGBTRIPLE) * clrUsed)) )
			return false;

	}

	/* \note see DIBitmap documentation for an explanation of this formula */
	DWORD size = (((( width * bitCount ) + 31) & ~31) >> 3) * height;

	fOk = !WriteFile(hPipe,
			imageBits,
			size,
			&numBytesWritten,
			NULL);

	if( fOk || (numBytesWritten != size) )
		return false;

	return true;
}
