// DIBitmap.h: interface for the DIBitmap class
/*! \file DIBitmap.h 
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIBITMAP_H__0CD63B7F_6A96_49D0_AC45_F08FC9CC900F__INCLUDED_)
#define AFX_DIBITMAP_H__0CD63B7F_6A96_49D0_AC45_F08FC9CC900F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*! @class DIBitmap
    @brief A class for loading and saving DIBs
    @ingroup win2vcc 
    A class for loading and saving Device Independent Bitmaps. This class is capable 
    of writing and reading windows .bmp files to/from a packed BITMAPINFO structure. \n
    See microsofts documentation on BITMAPINFOHEADER and BITMAPINFO structures.
*/
class DIBitmap  
{
public:
	bool saveFile(PCTSTR, const DWORD = CREATE_ALWAYS) const;
	bool loadFile(PCTSTR);

	static bool isCompatibleBitmap(const PBITMAPINFOHEADER);

	bool createCompatible(const int, const int, const int);
	bool set(const PBITMAPINFO);

	const DIBitmap &operator=(const DIBitmap&);
	
	/*! 
	 *	Wipes the Image
	 */
	void clear() { imageMemDealloc(); } 

    /*!
     * Returns a pointer to a Packed Bitmap i.e. a BITMAPINFO structure 
	 * with the image bits immediately after
     *
     * @return const PBITMAPINFO  : 
     */
	const PBITMAPINFO getPBITMAPINFO() const { return (BITMAPINFO*) this->bmInfoHeader; }

    /*!
     * Returns a pointer to a BITMAPINFOHEADER.  Provided for easy access to information
	 * about the contained bitmap
     *
     * @return const PBITMAPINFOHEADER  : 
     */
	const PBITMAPINFOHEADER getPBITMAPINFOHEADER() const { return this->bmInfoHeader; }

    /*!
     * Returns a pointer to the images bits
     *
     * @return BYTE*  : A pointer to an array of bytes 
	 * getPBITMAPINFOHEADER()->biSizeImage long.
     */
	BYTE* getPBytes() const { return this->imageBytes; }

    /*!
     * Returns a pointer to the images Palette
     *
     * @return RGBQUAD*  : A pointer to an array of RGBQUADS
	 * getPBITMAPINFOHEADER()->biClrUsed long.
     */
	RGBQUAD* getPPalette() const { return this->RGBQuads; }

    /*!
     * Overloaded cast function for statically casting the class to a pointer to a
	 * BITMAPINFO structure
     *
     * @return PBITMAPINFO  : 
     */
	operator PBITMAPINFO () const { return (BITMAPINFO*) this->bmInfoHeader; }

	DIBitmap();
	DIBitmap(const DIBitmap &);
	virtual ~DIBitmap();
private:
	imageMemDealloc();
	bool imageMemAlloc(const PBITMAPINFOHEADER);

	BITMAPINFOHEADER* bmInfoHeader; //!< pointer to BITMAPINFOHEADER
	RGBQUAD* RGBQuads;		//!< pointer to RGB Quads
	BYTE* imageBytes;		//!< pointer to the image bytes
	BYTE* assignedMemory;	//!< pointer to the assigned memory
};

#endif // !defined(AFX_DIBITMAP_H__0CD63B7F_6A96_49D0_AC45_F08FC9CC900F__INCLUDED_)
