// TString.h: interface for the TString class.
/*! \file TString.h 
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TSTRING_H__7F94F4C0_3291_11D6_8DF5_00C04F914030__INCLUDED_)
#define AFX_TSTRING_H__7F94F4C0_3291_11D6_8DF5_00C04F914030__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*! @class TString
 *  @brief Yet another string class
 *  @ingroup win2vcc 
 *  TString is designed to allow the use of both ANSI and Unicode Strings.
 *  It can be compiled in either an ANSI or Unicode environment and converts
 *  as necessary to the native string format.  The =, + and += operators are 
 *  overloaded to assign, concatenate and append.
 */
class TString  
{
	//@{
	/*! Overloading of the + operator to perform the concatenation of two strings */
	friend TString operator +( const TString&, const TString& );
	friend TString operator +( const TString&, PCSTR );
	friend TString operator +( const TString&, PCWSTR );
	friend TString operator +( PCSTR, const TString& );
	friend TString operator +( PCWSTR, const TString& );
	friend TString operator +( const TString&, const WCHAR );
	friend TString operator +( const TString&, const CHAR );
	friend TString operator +( const WCHAR, const TString& );
	friend TString operator +( const CHAR, const TString& );
	//@}
public:
	bool set(PCSTR);
	bool set(PCWSTR);
	bool set(const WCHAR);
	bool set(const CHAR);
	bool set(const int);

	int getBufferSize() const;
    int getLength() const;

	PCTSTR get() const;
    //! Overloaded static cast operator
    /*! Allows a TString to be used anywhere a constant String is expected */ 
	operator PCTSTR () const { return get(); }

	TCHAR operator []( const int ) const;

	const TString &operator =(const TString&);
	const TString &operator =(PCSTR);
	const TString &operator =(PCWSTR);
	const TString &operator =(const WCHAR);
	const TString &operator =(const CHAR);

	const TString &operator +=( const TString& );
	const TString &operator +=( PCSTR );
	const TString &operator +=( PCWSTR );
	const TString &operator +=( const WCHAR );
	const TString &operator +=( const CHAR );

	bool operator !() const;

	clear();

    TString(const WCHAR);
    TString(const CHAR);
    explicit TString(const int);

    TString(PCSTR);
    TString(PCWSTR);

    TString(const TString&);

	TString();
	virtual ~TString();

private:
	static PTSTR EMPTYSTRING;
    PTSTR string;				//!< The actual string

    int bufferLength;			//!< string length + null terminator

    inline int stringCopy(PCSTR, const int = -1);
    inline int stringCopy(PCWSTR, const int = -1);

    inline int memoryAlloc(PCSTR, const bool = true);
    inline int memoryAlloc(PCWSTR, const bool = true);
	inline bool alloc(const int);

    inline memoryDealloc();
};

#endif // !defined(AFX_TSTRING_H__7F94F4C0_3291_11D6_8DF5_00C04F914030__INCLUDED_)
