// WaitCallback.h: interface for the WaitCallback class.
/*! \file WaitCallback.h
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WAITCALLBACK_H__C554D63C_1250_49FC_9409_E6758E44402E__INCLUDED_)
#define AFX_WAITCALLBACK_H__C554D63C_1250_49FC_9409_E6758E44402E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*! Callback fuction type.
 * prototype: void callback(void* param, bool timedOut)
 * two parameters a void pointer and a boolean value, true 
 * iff wait timed out.
 */
typedef void (__stdcall *WAITCALLBACK) (void *, bool);

/*! @class WaitCallback
 *  @brief A class which implements most of the Win2000 function functionality
 *  @ingroup win2vcc 
 *  When an event becomes signaled the callback function registered is called.
 *  This is implemented fully in Windows 2000 but not NT 4 (hence this 
 *  implementation).  Refer to MS Windows Platform SDK for details.
 */
class WaitCallback  
{
public:
	bool hasWaitRegistered() const;
	void unregisterWait();
	bool registerWaitForSingleObject(HANDLE, const WAITCALLBACK, const PVOID, const DWORD, const bool = true);
	
	WaitCallback();
	virtual ~WaitCallback();
private:
	static DWORD WINAPI waitThread(LPVOID);

	HANDLE hWaitThread;		//!< Handle to the wait/worker thread
	bool justOnce;			//!< Boolean true if wait once
	DWORD timeOut;			//!< Wait Timeout
	HANDLE waitEvents[2];	//!< The waitable events
	WAITCALLBACK cBack;		//!< Pointer to the callback function
	PVOID cBackParam;		//!< Pointer to the callback function parameter

};

#endif // !defined(AFX_WAITCALLBACK_H__C554D63C_1250_49FC_9409_E6758E44402E__INCLUDED_)
