// BitmapScreen.h: interface for the BitmapScreen class.
/*! \file BitmapScreen.h
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BITMAPSCREEN_H__755F26C9_E871_40DE_B6E4_60A352FE8DCF__INCLUDED_)
#define AFX_BITMAPSCREEN_H__755F26C9_E871_40DE_B6E4_60A352FE8DCF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VScreen.h"

/*!
 * \class BitmapScreen
 * \brief A VScreen implemented as a bitmap
 * \ingroup winDressing
 * Provides a double buffered di bitmap 'screen' with read and
 * write locks.  The bitmap being read is the 'currentBuffer',
 * many simultaneous reads can be carried out.  Writes are performed 
 * on the other buffer, writes are serialised and requests are blocking
 * until reads have completed.
 */
class BitmapScreen : public VScreen  
{
public:
	DIBitmap* getWriteBuffer();
	bool DoneWritting();
	
	const DIBitmap* getReadBuffer();
	void DoneReading();

	void paint(HDC);

	BitmapScreen(const int, const int, const int, const int);
private:
	DIBitmap buffer[2];		//!< Two DIBitmaps forming a double buffer

	HANDLE writeMutex;		//!< A Handle to a Mutex used for serialising calls to getWriteBuffer()

	CSWMRG currentBufferLock; //!< A single writer multiple reader guard protecting the switch between buffers
	volatile int currentBuffer;	//!< An index into the array of DIBitmaps
};

#endif // !defined(AFX_BITMAPSCREEN_H__755F26C9_E871_40DE_B6E4_60A352FE8DCF__INCLUDED_)
