// ScreenPipe.h: interface for the ScreenPipe class.
/*! \file ScreenPipe.h
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCREENPIPE_H__8F28C996_E78E_4DDD_A034_DDD253AF41A7__INCLUDED_)
#define AFX_SCREENPIPE_H__8F28C996_E78E_4DDD_A034_DDD253AF41A7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define SP_UNKNOWN_ERROR			0L	//!< General Error
#define SP_SUCCESS					1L	//!< Success
#define SP_NOT_CONNECTED			2L	//!< No client connected
#define SP_ERROR_CREATING_BITMAP	3L	//!< Error reallocating DIBitmaps memory
#define SP_INVALID_DATA				4L	//!< Pipe data is invalid

/*!
 * \class ScreenPipe
 * \brief A server for reading screen buffers from VCC
 * \ingroup winDressing
 * This object is responsible for reading screen buffer 
 * data from the pipe and setting a DIBitmap.
 */
class ScreenPipe : public PServer  
{
public:
	DWORD readToDIB(DIBitmap&);

	ScreenPipe(const TString &);

private:
	inline DWORD readError();

	//! The structure of the header for each screen buffer
	/*!
	 * This is the first information on the pipe for each screen
	 * buffer.  The size of the message is calculated from this information
	 */
	typedef struct screenInfo {
		int hdrSize;	//!< The size of this structure
		int bitCount;	//!< The bpp of the image
		int clrUsed;	//!< The number of colours used
		int width;		//!< The width
		int height;		//!< The signed height
	} screenInfo;

};



#endif // !defined(AFX_SCREENPIPE_H__8F28C996_E78E_4DDD_A034_DDD253AF41A7__INCLUDED_)
