// UpdateHandler.h: interface for the UpdateHandler class.
/*! \file UpdateHandler.h
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_UPDATEHANDLER_H__39CF56A7_9BB9_4D07_B4B2_B875EED3D2CF__INCLUDED_)
#define AFX_UPDATEHANDLER_H__39CF56A7_9BB9_4D07_B4B2_B875EED3D2CF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BitmapScreen.h"
#include "ScreenPipe.h"

/*!
 * \class UpdateHandler
 * \brief An object running a separate update thread
 * \ingroup winDressing
 * The updater reads new screens from the pipe and writes them to
 * the BitmapScreen.  Depending on capture settings the bitmap is also 
 * written to disk.
 */
class UpdateHandler  
{
public:
	bool setCapture(const TString&, const TString&, const int, const int);
    bool setCaptureName(const TString&);
    bool setCaptureDir(const TString&);
    bool setCaptureLimit(const int);
    bool setCaptureFrequency(const int);
	void disableCapture();
	bool enableCapture();

	//! get the capture name TString
    const TString getCaptureName() const { return captureName; }
	//! get the capture directory TString
    const TString getCaptureDir() const { return captureDir; }
	//! get the capture limit
    int getCaptureLimit() const { return captureLimit; }
	//! get the cpature Frequency
    int getCaptureFrequency() const { return captureEvery; }

	void stop(const bool = true);
	bool run();

	UpdateHandler(ScreenPipe&, BitmapScreen&, HWND);
	virtual ~UpdateHandler();
private:
	HANDLE hUpdateThread;	//!< Handle to the update thread
	static DWORD WINAPI update(LPVOID);

	volatile bool exit;		//!< true when update thread should terminate

	BitmapScreen *screen;	//!< Pointer to the BitmapScreen
	ScreenPipe *pipe;		//!< Pointer to the ScreenPipe
	const HWND hWnd;		//!< Handle to the main window

    HANDLE captureMutex;	//!< A Handle to a Mutex protecting the capture details during capture
    TString captureName;	//!< The base name (to which the frame number is appended) of the .bmp file
    TString captureDir;		//!< The directory for storing the captures
    int captureEvery;		//!< The capture Frequency
    int captureLimit;		//!< The number of captures to perform
	volatile bool capture;	//!< Boolean indicating that screen captures should be performed
};

#endif // !defined(AFX_UPDATEHANDLER_H__39CF56A7_9BB9_4D07_B4B2_B875EED3D2CF__INCLUDED_)
