// VScreen.cpp: implementation of the VScreen class.
/*! \file VScreen.cpp
*/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VScreen.h"

//////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////

/*!
 * \note passing a negative height moves the origin to the bottom left
 * as apposed to the top left if the height is positive.
 * @param tlX : top left x co-ordinate of the (rectangular) screen
 * @param tlY : top left y co-ordinate
 * @param w : width of the screen
 * @param h : signed height of the screen
 */
VScreen::VScreen(const int tlX, const int tlY, const int w, const int h)
{
	pointX = 0;
	pointY = 0;
	width = abs(w);	// ensure positive width
	height = h;
	moveScreen(tlX, tlY);
}

//////////////////////////////////////////////////////////////////////
// Member Functions
//////////////////////////////////////////////////////////////////////

/*!
 * Move the screen top left corner to the specified x, y co-ordinate
 */
void VScreen::moveScreen(const int tlX, const int tlY)
{
	screenRgn = CreateRectRgn(
		tlX, tlY, 
		this->width + tlX, 
		abs(height) + tlY);

	GetRgnBox(screenRgn, &screenRect);
}

/*!
 * This function takes a window co-ordinate and tests if it is in the 
 * screen region. The internal point is set to a VScreen co-ordinate 
 * iff it is.
 *
 * @param x : window x co-ordinate
 * @param y : window y co-ordinate
 *
 * @return bool : true iff point is on 'screen'
 */
bool VScreen::PointOnScreen(const int x, const int y)
{
	if ( PtInRegion(screenRgn, x, y) ){
		pointX = x - screenRect.left;

		if( height < 0 )
			pointY = -y + screenRect.top - (height + 1);
		else
			pointY = y - screenRect.top;

		return true;
	} else {
		return false;
	}
}

/*!
 * Gets the regions Rectangle (see MS Platform SDK)
 */
RECT VScreen::getRgnRect() const
{
	return screenRect;
}

/*!
 * Gets the region (see MS Platform SDK)
 */
HRGN VScreen::getRgn() const
{
	return screenRgn;
}

/*!
 * Invalidates the entire region (marks for painting) and sends a WM_PAINT message
 */
void VScreen::invalidate(HWND hWnd, bool Erase)
{
	InvalidateRgn(hWnd, screenRgn, Erase);
}

/*!
 * Returns the x co-ordinate of the last point on the screen
 */
int VScreen::getScreenX() const
{
	return pointX;
}

/*!
 * Returns the y co-ordinate of the last point on the screen
 */
int VScreen::getScreenY() const
{
	return pointY;
}

/*!
 * Returns the (signed) height of the VScreen
 */
int VScreen::getHeight() const
{
	return height;
}

/*!
 * Returns the width of the VScreen
 */
int VScreen::getWidth() const
{
	return width;
}
