// VScreen.h: interface for the VScreen class.
/*! \file VScreen.h
*/
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VSCREEN_H__DC16767E_0866_4608_B96F_6A1FDB7FF0B7__INCLUDED_)
#define AFX_VSCREEN_H__DC16767E_0866_4608_B96F_6A1FDB7FF0B7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*!
 * \class VScreen
 * \brief An abstract class for 'screens'
 * \ingroup winDressing
 * A virtual screen is a rectangular area in a window.  The actual screen
 * area inside this region doesn't have to be rectangular.  A virtual screen
 * can draw itself and has it's own co-ordinate system.
 */
class VScreen  
{
public:
	int getScreenX() const;
	int getScreenY() const;
	virtual bool PointOnScreen(const int, const int);

	virtual void paint(HDC) = 0;	//!< Pure Virual function, hence abstract
	void invalidate(HWND, bool = false);

	HRGN getRgn() const;
	RECT getRgnRect() const;
	int getHeight() const;
	int getWidth() const;

    void moveScreen(const int, const int);

	VScreen(const int, const int, const int, const int);
private:
	int height;				//!< signed height of the screen
	int width;				//!< width of the screen
	int pointX;				//!< x co-ordinate of top left corner
	int pointY;				//!< y co-ordinate of top left corner
	HRGN screenRgn;			//!< The screens region
	RECT screenRect;		//!< The screens region as a RECT
};

#endif // !defined(AFX_VSCREEN_H__DC16767E_0866_4608_B96F_6A1FDB7FF0B7__INCLUDED_)
